"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var net = require("net");
var socketStream = require("socket.io-stream");
var socket_io_client_1 = require("socket.io-client");
var Client = (function () {
    function Client(options) {
        this.IDLE_SOCKET_TIMEOUT_MILLISECONDS = 1000 * 30;
        this.options = options;
        this.options.host = options.host || '127.0.0.1';
    }
    Client.prototype.getConnection = function () {
        var _this = this;
        return new Promise(function (res) {
            _this.socket = (0, socket_io_client_1.io)(_this.options.server, {
                transports: ['websocket'],
                secure: true,
                rejectUnauthorized: false,
                reconnectionDelayMax: 10000,
            });
            _this.socket.on('connect', _this.onConnect.bind(_this, res));
            _this.socket.on('incomingClient', _this.onIncomingClient.bind(_this));
        });
    };
    Client.prototype.onConnect = function (res) {
        console.log(new Date() + ': Connected!');
        console.log(new Date() +
            ": Requesting subdomain '".concat(this.options.subdomain, "' via ").concat(this.options['server']));
        this.socket.emit('createTunnel', this.options.subdomain);
        this.socket.on('createTunnel', this.onCreateTunnel.bind(this, res));
    };
    Client.prototype.onCreateTunnel = function (res, response) {
        var _a = this.options, server = _a.server, subdomain = _a.subdomain;
        var protocol = server.split('://')[0];
        var host = server.split('://')[1];
        var url = "".concat(protocol, "://").concat(subdomain, ".").concat(host);
        console.log(new Date() + ": ".concat(response, ". Please visit ").concat(url));
        res(url);
    };
    Client.prototype.onIncomingClient = function (clientID) {
        var _this = this;
        var client = net.connect(this.options.port, this.options.host, function () {
            var stream = socketStream.createStream();
            stream.pipe(client).pipe(stream);
            stream.on('end', function () {
                client.destroy();
            });
            socketStream(_this.socket).emit(clientID, stream);
        });
        client.setTimeout(this.IDLE_SOCKET_TIMEOUT_MILLISECONDS);
        client.on('timeout', function () {
            client.end();
        });
        client.on('error', function () {
            var stream = socketStream.createStream();
            socketStream(_this.socket).emit(clientID, stream);
            stream.end();
        });
    };
    return Client;
}());
exports.default = Client;
